/*
Joan Soler-Adillon 
 2017
 Adapted from examples in Shiffman's Kinect_V1 Library
 
 LLegeix la silueta de l'usuari entre les distàncies mínimes i màximes que especifiquem
 Lee la silueta del usuario entre las distancia mínimas y máximas especificadas
 */
//Importem la bilioteca per poder treballar amb KINECT
//Importamos la biblioteca para poder acceder a KINECT
import org.openkinect.freenect.*;
import org.openkinect.processing.*;

//Creem un objecte per accedir a la càmera
//Creamos un objecto para acceder a la camara
Kinect k;

//Establim el color amb el que dibuixarem la silueta
//Color para dibujar luego la silueta
color p = color(255, 0, 0);//set color to paint silhouette
//Màxim i mínim de distància dins les quals buscarem l'usuari
//Máximo y mínimo de distancia dentro de la cual busacaremos al usuario
int minThreshold = 500;
int maxThreshold = 600; 


void setup()
{
  //Fem coincidir la mida de pantalla amb la resolució de KINECT
  //Tamaño de ventana es el mismo que la resoución de KINECT 
  size(640, 480);
  //Instanciem KINECT
  //Instanciamos KINECT
  k = new Kinect(this);
  //...i li diem que volem accedir a la info de profunditat
  //...y activamos acceso a la imágen de profundidad
  k.initDepth();
  //i el vídeo RGB
  //y el vídeo RGB
  k.initVideo();
}
void draw()
{
  //Fons / fondo
  background(0);

  //Si cliquem, que es vegi la imatge real
  //Si clicamos, que se vea la imágen real
  if (mousePressed) {
    image(k.getVideoImage(), 0, 0);//show rgb image
  }
  //Creem un array y hi posem la info de profunditat
  //Creamos un array para guardar la info de profundidad
  int[] depthValues = k.getRawDepth();
  //...i variables que seran per marcar el rectangle que marca la silueta, que instanciem fora pla
  //..y variables para marcar el rectángulo que marca la silueta, instanciadas fuera pantalla
  int topY = -1;
  int lowY = width+1;
  int leftX = -1; 
  int rightX = height+1; 

  //Per saber si almenys hi ha un píxel amb usuari
  //para marcar si hay almenos un píxel ocupado
  boolean onePx = false;
  //I fem el doble loop per visitar tots els pixels de la pantalla
  //Doble loop para recorrer todos los píxels
  for (int x=0; x<width; x++) {
    for (int y=0; y<height; y++) {
      //Mapegem les coordenades a la posició dins l'array
      //Mapeamos coordenadas a posición en el array
      int currentPosition = x + (y * 640);
      //I en llegim el valor
      //Y leemos el valor
      int currentDepth = depthValues[currentPosition]; 

      //Exploent 0, que és massa aprop o lluny... 
      //Excluyendo 0, que es demasiado cerca o lejos (hola Epi, Hola Blas!)
      if (currentDepth !=0) {
        //Mirem si estem dins l'espai que ens interessa
        //Miramos si estamos dentro del espacio que nos interesa
        if (currentDepth > minThreshold && currentDepth < maxThreshold) {
          //I si hi som, pintem aquest pixel del color especificat a l'inici
          //Y si lo estamos, pintamos este píxel en concreto del color p (decidido arriba)
          set(x, y, p);
          //I recordem que almenys n'hem trobat un
          //Y marcamos que hay un píxel ocupado
          onePx = true;
          //I ara mirem si és el píxel més amunt, o més avall, o més a la dreta o esquera trobat fins ara
          //Y Miramos si es el píxel más arriba, abajos, izquierda o derecha encontrado hasta ahora
          if (x > leftX)leftX = x;
          else if (x < rightX)rightX = x;
          if (y > topY)topY = y;
          else if (y < lowY)lowY = y;
        }
      }
    }//end of inner double loop
  }//end of double loop

  //I finalment, si hi ha un o més píxels ocupats, dibuixem la caixa contenidora...
  //Y finalmente, si almenos hay un píxel ocupado, dibujamos la caja contenedora...
  if (onePx) {
    rectMode(CORNERS);
    noFill();
    stroke(0, 255, 0);
    rect(leftX, topY, rightX, lowY);

    //... i el centre de la caixa (centre de gravetat)
    //...y el centro de la caja (centro de gravedad)
    fill(255, 255, 0);
    noStroke();
    int centerX = leftX + (rightX - leftX)/2;
    int centerY = lowY + (topY - lowY)/2;
    ellipse(centerX, centerY, 20, 20);
  }
}